*******************************************************************************
*   File........: acount3d.prg
*   Date........: October 5, 1993
*   Author......: Ly Minh Tr (CIS 70641,3127)
*   Copyright...: None...freeware!
*   Version.....: 1.02
*******************************************************************************
*   PURPOSE:    acount3d() counts the number of times expression <cexpr> exists
*               in the 3D-array <parray>
*
*   SYNTAX:     acount3d(<parray>, <pexpr> [, <nloc>])
*
*   PARAMETERS: <parray>    = the 3D-array to be indexed
*               <pexpr>     = the expression to count
*               <nloc>      = if > 0, then acount3d() returns the index of
*                             the nth occurrence of <pexpr>
*
*   RETURNS:    Numeric
*
*   REMARKS:    Acount3d() calls Foxpro's ASCAN() to determine the starting
*               point where the counting begins.  Thus, similar to ASCAN(),
*               the number of matches is based on the state of SET EXACT.
*
*   SEE ALSO:   adel3d(), aelem3d(), adim3d(), afill3d(), ains3d(), alen3d(),
*               aloc3d(), astore3d(), asubs3d()
*******************************************************************************
PARAMETER parray, pexpr, nloc
PRIVATE nval, ncount, i, nstart, nlen, cexprtype
EXTERNAL ARRAY parray

DO CASE
    CASE PARAMETER() < 2
        WAIT WINDOW "Invalid number of parameters given!" TIMEOUT 1
        RETURN 0

    CASE (PARAMETER() = 3) AND (TYPE([nloc]) <> [N])
        WAIT WINDOW "Invalid 3rd parameter given!" TIMEOUT 1
        RETURN 0

    *
    * First element of a 3D-array contains the array ID and hold the
    * dimensions of the array.  If this is not true, then the array is
    * not a 3D-array!
    *
    CASE TYPE("parray[1]") <> "C" OR !([3D] $ parray[1])
        WAIT WINDOW "Given array is not a 3D-array!" TIMEOUT 1
        RETURN 0
ENDCASE

nval        = 0
nstart      = ASCAN(parray, pexpr)
cexprtype   = TYPE([pexpr])

IF nstart > 0
    IF (TYPE([nloc]) == [N])
        IF (nloc = 0)
            RETURN 0
        ELSE
            nlen = nstart + nloc - 1
        ENDIF
    ELSE
        nlen = ALEN(parray)
    ENDIF

    FOR i = nstart TO nlen
        IF (TYPE("parray[i]") == cexprtype) AND (parray[i] = pexpr)
            nval = nval + 1
        ENDIF
    ENDFOR
	
    IF (TYPE([nloc]) == [N])
    	nval = i - 1
    ENDIF
ENDIF

RETURN nval
