*******************************************************************************
*   File........: adel3d.prg
*   Date........: September 30, 1993
*   Author......: Ly Minh Tr (CIS 70641,3127)
*   Copyright...: None...freeware!
*   Version.....: 1.02
*******************************************************************************
*   PURPOSE:    Deletes a plane from a 3D-array
*
*   SYNTAX:     adel3d(<parray>, <nplane> [, <naxis])
*
*   PARAMETERS: <parray>    = the 3D-array to delete data from
*               <nplane>    = plane to delete data from
*               <naxis>     = 0 or empty to delete plane on x-axis
*                           = 1 to delete plane y-axis
*                           = 2 to delete plane z-axis
*
*   RETURNS:    Numeric     = 1 if successful, 0 if deletion fails
*
*   REMARKS:    Mimics Foxpro's ADEL() for a 3D-array.
*
*   SEE ALSO:   acount3d(), adim3d(), aelem3d(), ains3d(), afill3d(), alen3d(),
*               aloc3d(), astore3d(), asubs3d()
*******************************************************************************
PARAMETERS parray, nplane, naxis
PRIVATE nval, nloc, nrow, ncol, nwidth, i, j, k
EXTERNAL ARRAY parray

nval    = 0
naxis   = IIF(EMPTY(naxis), 0, naxis)

DO CASE
    CASE !INLIST(PARAMETER(), 2, 3)
        WAIT WINDOW "Invalid number of parameters given!" TIMEOUT 1

    CASE TYPE("parray[1]") <> "C" OR !([3D] $ parray[1])
        WAIT WINDOW "Given array is not a 3D-array!"

    CASE TYPE("nplane") <> "N"
        WAIT WINDOW "Invalid plane subscript!" TIMEOUT 1

    CASE !INLIST(naxis, 0, 1, 2)
        WAIT WINDOW "Invalid axis specifier!" TIMEOUT 1

    OTHERWISE
        nval = 1
ENDCASE

IF nval = 1
    nrow    = INT(VAL(SUBSTR(parray[1], 4)))
    ncol    = INT(VAL(SUBSTR(parray[1], AT(",", parray[1], 2)+1)))
    nwidth  = INT(VAL(SUBSTR(parray[1], AT(",", parray[1], 3)+1)))

    DO CASE
        CASE naxis == 0
            IF nplane > nrow
                WAIT WINDOW [Invalid x-axis plane subscript given!] TIMEOUT 1
                nval = 0
            ELSE
                nloc = aloc3d(@parray, nplane, 1, 1)
                nlen = nloc + (ncol * nwidth) - 1

                FOR i = nlen TO nloc STEP -1
                    = ADEL(parray, i)
                ENDFOR
            ENDIF

        CASE naxis == 1
            IF nplane > ncol
                WAIT WINDOW [Invalid y-axis plane subscript given!] TIMEOUT 1
                nval = 0
            ELSE
                FOR i = nrow TO 1 STEP -1
                    nloc = aloc3d(@parray, i, nplane, 1)
                    nlen = nloc + nwidth - 1

                    FOR j = nlen TO nloc STEP -1
                        = ADEL(parray, j)
                    ENDFOR
                ENDFOR
           ENDIF

        CASE naxis == 2
            IF nplane > nwidth
                WAIT WINDOW [Invalid z-axis plane subscript given!] TIMEOUT 1
                nval = 0
            ELSE
                FOR i = nrow TO 1 STEP -1
                    FOR j = ncol TO 1 STEP -1
                        nloc = aloc3d(@parray, i, j, nplane)
                        = ADEL(parray, nloc)
                    ENDFOR
                ENDFOR
            ENDIF
    ENDCASE
ENDIF

RETURN nval
