*******************************************************************************
*   File........: adim3d.prg
*   Date........: September 30, 1993
*   Author......: Ly Minh Tr (CIS 70641,3127)
*   Copyright...: None...freeware!
*   Version.....: 1.02
*******************************************************************************
*   PURPOSE:    adim3d() creates a 3D-array via a linear representation.  It
*               requires 4 parameters.  The first is the name of the array
*               (which must be already declared by the calling program) and
*               the last 3 are the dimensions of the 3D-array.
*
*   SYNTAX:     adim3d(<parray>, <nrow>, <ncol>, <nwidth>)
*
*   PARAMETERS: <parray>    = the 3D-array to be created
*               <nrow>      = # of elements in dimension 1
*               <ncol>      = # of elements in dimension 2
*               <nwidth>    = # of elements in dimension 3
*
*   RETURNS:    Boolean     = .T. if successful, .F. if array was not created
*
*   REMARKS:    An extra element is created by this function to store the
*               dimensions of the 3D-array and the array 3D indicator.
*
*   SEE ALSO:   acount3d(), adel3d(), aelem3d(), ains3d(), afill3d(), alen3d(),
*               aloc3d(), astore3d(), asubs3d()
*******************************************************************************
PARAMETER parray, nrow, ncol, nwidth
PRIVATE i, nsize

DO CASE
    CASE PARAMETER() <> 4
        WAIT WINDOW "Invalid number of parameters given!" TIMEOUT 1
        RETURN .F.

    CASE TYPE("parray[1]") == "U"
        WAIT WINDOW "Array " + parray + " has not been declared!"
        RETURN .F.

    CASE (TYPE("nrow") <> "N") OR (nrow < 1)
        WAIT WINDOW "Invalid row value given!" TIMEOUT 1
        RETURN .F.

    CASE (TYPE("ncol") <> "N") OR (ncol < 1)
        WAIT WINDOW "Invalid column value given!" TIMEOUT 1
        RETURN .F.

    CASE (TYPE("nwidth") <> "N") OR (nwidth < 1)
        WAIT WINDOW "Invalid width value given!" TIMEOUT 1
        RETURN .F.
ENDCASE

*
* Determine the number of elements in the 3D-array
*
nsize   = nrow * ncol * nwidth + 1          && Add 1 extra for array header
DIMENSION parray[nsize]                     && Declare the 3D-array

*
* Was it created successfully?  This is determined by testing for the type
* of the nsizeth element in the array.  If its undefined, then the array
* failed during creation.
*
IF TYPE("parray[" + STR(nsize) + "]") == "U"
    DIMENSION parray[1]
    WAIT WINDOW "Not enough memory to create array!" TIMEOUT 1
    RETURN .F.
ENDIF

*
* The first element of array is reserved for its dimensions and ID!
*
parray[1] = "3D," + ALLTRIM(STR(nrow)) + "," + ;
             ALLTRIM(STR(ncol)) + "," + ALLTRIM(STR(nwidth))

RETURN .T.
